@@ -44,7 +44,8 @@
          :reader acceptor-port
          :documentation "The port the acceptor is listening on.  The
 default is 80.  Note that depending on your operating system you might
-need special privileges to listen on port 80.")
+need special privileges to listen on port 80.  When 0, the port will be
+chosen by the system the first time the acceptor is started.")
    (address :initarg :address
             :reader acceptor-address
             :documentation "The address the acceptor is listening on.
@@ -120,7 +121,7 @@ process different from the one where START was called.")
                   :accessor acceptor-listen-socket
                   :documentation "The socket listening for incoming
 connections.")
-   #-:lispworks
+   #-(or :lispworks4 :lispworks5 :lispworks6)
    (listen-backlog :initarg :listen-backlog
                    :reader acceptor-listen-backlog
                    :documentation "Number of pending connections
@@ -131,7 +132,7 @@ connections.")
                         :documentation "A flag that makes the acceptor
 shutdown itself when set to something other than NIL.")
    (requests-in-progress :initform 0
-                         :accessor accessor-requests-in-progress
+                         :accessor acceptor-requests-in-progress
                          :documentation "The number of
 requests currently in progress.")
    (shutdown-queue :initform (make-condition-variable)
@@ -164,7 +165,7 @@ stream or to NIL to suppress logging.")
  messages.  Files must be named <return-code>.html with <return-code>
  representing the HTTP return code that the file applies to,
  i.e. 404.html would be used as the content for a HTTP 404 Not found
- response.")
+ response. Set *SHOW-LISP-ERRORS-P* to T for this setting to work.")
    (document-root :initarg :document-root
                   :accessor acceptor-document-root
                   :documentation "Directory pathname that points to
@@ -176,7 +177,8 @@ acceptor-dispatch-request method handles the request."))
    :name (gensym)
    :request-class 'request
    :reply-class 'reply
-   #-lispworks :listen-backlog #-lispworks 50
+   #-(or :lispworks4 :lispworks5 :lispworks6) :listen-backlog
+   #-(or :lispworks4 :lispworks5 :lispworks6) 50
    :taskmaster (make-instance (cond (*supports-threads-p* 'one-thread-per-connection-taskmaster)
                                     (t 'single-threaded-taskmaster)))
    :output-chunking-p t
@@ -251,7 +253,7 @@ or similar)."))
   (:documentation "In a loop, accepts a connection and hands it over
 to the acceptor's taskmaster for processing using
 HANDLE-INCOMING-CONNECTION.  On LispWorks, this function returns
-immediately, on other Lisps it retusn only once the acceptor has been
+immediately, on other Lisps it returns only once the acceptor has been
 stopped."))
 
 (defgeneric initialize-connection-stream (acceptor stream)
@@ -308,17 +310,20 @@ they're using secure connections - see the SSL-ACCEPTOR class."))
 
 (defmethod start ((acceptor acceptor))
   (setf (acceptor-shutdown-p acceptor) nil)
-  (start-listening acceptor)
   (let ((taskmaster (acceptor-taskmaster acceptor)))
     (setf (taskmaster-acceptor taskmaster) acceptor)
+    (start-listening acceptor)
     (execute-acceptor taskmaster))
   acceptor)
 
 (defmethod stop ((acceptor acceptor) &key soft)
-  (setf (acceptor-shutdown-p acceptor) t)
+  (with-lock-held ((acceptor-shutdown-lock acceptor))
+    (setf (acceptor-shutdown-p acceptor) t))
+  #-lispworks
+  (wake-acceptor-for-shutdown acceptor)
   (when soft
     (with-lock-held ((acceptor-shutdown-lock acceptor))
-      (when (plusp (accessor-requests-in-progress acceptor))
+      (when (plusp (acceptor-requests-in-progress acceptor))
         (condition-variable-wait (acceptor-shutdown-queue acceptor)
                                  (acceptor-shutdown-lock acceptor)))))
   (shutdown (acceptor-taskmaster acceptor))
@@ -330,6 +335,29 @@ they're using secure connections - see the SSL-ACCEPTOR class."))
   (mp:process-kill (acceptor-process acceptor))
   acceptor)
 
+#-lispworks
+(defun wake-acceptor-for-shutdown (acceptor)
+  "Creates a dummy connection to the acceptor, waking ACCEPT-CONNECTIONS while it is waiting.
+This is supposed to force a check of ACCEPTOR-SHUTDOWN-P."
+  (handler-case
+      (multiple-value-bind (address port) (usocket:get-local-name (acceptor-listen-socket acceptor))
+        (let ((conn (usocket:socket-connect
+                     (cond
+                       ;; 0.0.0.0/8 is invalid as the target for a
+                       ;; connection, so we have to use something
+                       ;; else.
+                       ((and (= (length address) 4) (zerop (elt address 0)))
+                        #(127 0 0 1))
+                       ;; In IPv6, :: is similarly reserved.
+                       ((and (= (length address) 16)
+                             (every #'zerop address))
+                        #(0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1))
+                       (t address))
+                     port)))
+          (usocket:socket-close conn)))
+    (error (e)
+      (acceptor-log-message acceptor :error "Wake-for-shutdown connect failed: ~A" e))))
+
 (defmethod initialize-connection-stream ((acceptor acceptor) stream)
  ;; default method does nothing
  stream)
@@ -354,11 +382,11 @@ they're using secure connections - see the SSL-ACCEPTOR class."))
 
 (defun do-with-acceptor-request-count-incremented (*acceptor* function)
   (with-lock-held ((acceptor-shutdown-lock *acceptor*))
-    (incf (accessor-requests-in-progress *acceptor*)))
+    (incf (acceptor-requests-in-progress *acceptor*)))
   (unwind-protect
        (funcall function)
     (with-lock-held ((acceptor-shutdown-lock *acceptor*))
-      (decf (accessor-requests-in-progress *acceptor*))
+      (decf (acceptor-requests-in-progress *acceptor*))
       (when (acceptor-shutdown-p *acceptor*)
         (condition-variable-signal (acceptor-shutdown-queue *acceptor*))))))
 
@@ -376,14 +404,20 @@ they're using secure connections - see the SSL-ACCEPTOR class."))
                                   content-stream
                                   method
                                   uri
+                                  remote
+                                  local
                                   server-protocol)
   "Make a REQUEST instance for the ACCEPTOR, setting up those slots
   that are determined from the SOCKET by calling the appropriate
   socket query functions."
   (multiple-value-bind (remote-addr remote-port)
-      (get-peer-address-and-port socket)
+      (if remote
+          (values-list remote)
+          (get-peer-address-and-port socket))
     (multiple-value-bind (local-addr local-port)
-        (get-local-address-and-port socket)
+        (if local
+            (values-list local)
+            (get-local-address-and-port socket))
       (make-instance (acceptor-request-class acceptor)
                      :acceptor acceptor
                      :local-addr local-addr
@@ -412,7 +446,9 @@ they're using secure connections - see the SSL-ACCEPTOR class."))
 (defmethod process-connection ((*acceptor* acceptor) (socket t))
   (let* ((socket-stream (make-socket-stream socket *acceptor*))
          (*hunchentoot-stream*)
-         (*close-hunchentoot-stream* t))
+         (*close-hunchentoot-stream* t)
+         (remote (multiple-value-list (get-peer-address-and-port socket)))
+         (local (multiple-value-list (get-local-address-and-port socket))))
     (unwind-protect
          ;; process requests until either the acceptor is shut down,
          ;; *CLOSE-HUNCHENTOOT-STREAM* has been set to T by the
@@ -449,6 +485,8 @@ chunked encoding, but acceptor is configured to not use it.")))))
                                                               :content-stream *hunchentoot-stream*
                                                               :method method
                                                               :uri url-string
+                                                              :remote remote
+                                                              :local local
                                                               :server-protocol protocol))))
                   (finish-output *hunchentoot-stream*)
                   (setq *hunchentoot-stream* (reset-connection-stream *acceptor* *hunchentoot-stream*))
@@ -459,9 +497,9 @@ chunked encoding, but acceptor is configured to not use it.")))))
                  ;; as we are at the end of the request here, we ignore all
                  ;; errors that may occur while flushing and/or closing the
                  ;; stream.
-                 (ignore-errors*
+                 (ignore-errors
                   (finish-output stream))
-                 (ignore-errors*
+                 (ignore-errors
                   (close stream :abort t))))
           (unless (or (not *hunchentoot-stream*)
                       (eql socket-stream *hunchentoot-stream*))
@@ -546,13 +584,19 @@ catches during request processing."
                                :element-type '(unsigned-byte 8)))
   (values))
 
+#-:lispworks
+(defmethod start-listening :after ((acceptor acceptor))
+  (when (zerop (acceptor-port acceptor))
+    (setf (slot-value acceptor 'port) (usocket:get-local-port (acceptor-listen-socket acceptor)))))
+
 #-:lispworks
 (defmethod accept-connections ((acceptor acceptor))
   (usocket:with-server-socket (listener (acceptor-listen-socket acceptor))
     (loop
-     (when (acceptor-shutdown-p acceptor)
-       (return))
-     (when (usocket:wait-for-input listener :ready-only t :timeout +new-connection-wait-time+)
+      (with-lock-held ((acceptor-shutdown-lock acceptor))
+        (when (acceptor-shutdown-p acceptor)
+          (return)))
+      (when (usocket:wait-for-input listener :ready-only t)
        (when-let (client-connection
                   (handler-case (usocket:socket-accept listener)
                     ;; ignore condition
@@ -573,12 +617,21 @@ catches during request processing."
                             :process-name (format nil "Hunchentoot listener \(~A:~A)"
                                                   (or (acceptor-address acceptor) "*")
                                                   (acceptor-port acceptor))
+                            #-(or :lispworks4 :lispworks5 :lispworks6)
+                            :backlog
+                            #-(or :lispworks4 :lispworks5 :lispworks6)
+                            (acceptor-listen-backlog acceptor)
                             ;; this function is called once on startup - we
-                            ;; use it to check for errors
+                            ;; use it to check for errors and random port
                             :announce (lambda (socket &optional condition)
-                                        (declare (ignore socket))
                                         (when condition
-                                          (error condition)))
+                                          (error condition))
+                                        (when (or (null (acceptor-port acceptor))
+                                                  (zerop (acceptor-port acceptor)))
+                                          (multiple-value-bind (address port)
+                                              (comm:get-socket-address socket)
+                                            (declare (ignore address))
+                                            (setf (slot-value acceptor 'port) port))))
                             ;; this function is called whenever a connection
                             ;; is made
                             :function (lambda (handle)
