@@ -56,6 +56,12 @@ cookie expires \(or NIL).")
            :initform nil
            :accessor cookie-domain
            :documentation "The domain this cookie is valid for \(or NIL).")
+   (same-site :initarg :same-site
+           :initform nil
+           :accessor cookie-same-site
+           :documentation "The SameSite attribute for the cookie, needs
+to be one of \"None\", \"Lax\" or \"Strict\". Defaults to \"None\". See
+<https://datatracker.ietf.org/doc/html/draft-ietf-httpbis-rfc6265bis-02#section-5.3.7>.")
    (secure :initarg :secure
            :initform nil
            :accessor cookie-secure
@@ -91,7 +97,7 @@ REPLY object REPLY. If a cookie with the same name
         (push (cons name cookie) (cookies-out reply))
         cookie))))
 
-(defun set-cookie (name &key (value "") expires max-age path domain secure http-only (reply *reply*))
+(defun set-cookie (name &key (value "") expires max-age path domain same-site secure http-only (reply *reply*))
   "Creates a cookie object from the parameters provided and adds
 it to the outgoing cookies of the REPLY object REPLY. If a cookie
 with the name NAME \(case-sensitive) already exists, it is
@@ -103,6 +109,7 @@ replaced."
                               :max-age max-age
                               :path path
                               :domain domain
+                              :same-site same-site
                               :secure secure
                               :http-only http-only)
                reply))
@@ -116,12 +123,13 @@ replaced."
   "Converts the COOKIE object COOKIE to a string suitable for a
 'Set-Cookie' header to be sent to the client."
   (format nil
-          "~A=~A~@[; Expires=~A~]~@[; Max-Age=~A~]~@[; Domain=~A~]~@[; Path=~A~]~:[~;; Secure~]~:[~;; HttpOnly~]"
+          "~A=~A~@[; Expires=~A~]~@[; Max-Age=~A~]~@[; Domain=~A~]~@[; Path=~A~]~@[; SameSite=~A~]~:[~;; Secure~]~:[~;; HttpOnly~]"
           (cookie-name cookie)
           (cookie-value cookie)
           (cookie-date (cookie-expires cookie))
           (cookie-max-age cookie)
           (cookie-domain cookie)
           (cookie-path cookie)
+          (cookie-same-site cookie)
           (cookie-secure cookie)
           (cookie-http-only cookie)))
