@@ -1,7 +1,6 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HUNCHENTOOT; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/hunchentoot/mime-types.lisp,v 1.4 2008/02/13 16:02:18 edi Exp $
 
-;;; Copyright (c) 2004-2009, Dr. Edmund Weitz. All rights reserved.
+;;; Copyright (c) 2004-2010, Dr. Edmund Weitz. All rights reserved.
 
 ;;; Redistribution and use in source and binary forms, with or without
 ;;; modification, are permitted provided that the following conditions
@@ -29,7 +28,8 @@
 
 (in-package :hunchentoot)
 
-(defparameter *mime-type-list* '(("application/andrew-inset" "ez")
+(defparameter *mime-type-list* '(("application/wasm" "wasm")
+                                 ("application/andrew-inset" "ez")
                                  ("application/cu-seeme" "cu")
                                  ("application/dsptype" "tsp")
                                  ("application/futuresplash" "spl")
@@ -313,6 +313,7 @@
                                  ("text/x-dsrc" "d")
                                  ("text/x-haskell" "hs")
                                  ("text/x-java" "java")
+                                 ("text/javascript" "js")
                                  ("text/x-literate-haskell" "lhs")
                                  ("text/x-moc" "moc")
                                  ("text/x-pascal" "pp" "as")
@@ -334,6 +335,7 @@
                                  ("video/quicktime" "qt" "mov")
                                  ("video/vnd.mpegurl" "mxu")
                                  ("video/x-la-asf" "lsf" "lsx")
+                                 ("video/x-m4v" "m4v")
                                  ("video/x-mng" "mng")
                                  ("video/x-ms-asf" "asf" "asx")
                                  ("video/x-ms-wm" "wm")
@@ -359,4 +361,4 @@ of file suffixes for the corresponding type.")
   "Given a pathname designator PATHSPEC returns the MIME type
 \(as a string) corresponding to the suffix of the file denoted by
 PATHSPEC \(or NIL)."
-  (gethash (pathname-type pathspec) *mime-type-hash*))
\ No newline at end of file
+  (gethash (pathname-type pathspec) *mime-type-hash*))
